/**********************************************************

	Copyright (c) Dale Semchishen 1997
	All Rights Reserved

	set32.c

	Description:
		32 bit set manipulation library for all platforms

		This library consists of macros and functions that
		provide set manipulation services.

	Public macros and functions:
		set32_Assign()		- define all bits in a set
		set32_Not()			- invert all bits in a set
		set32_And()			- AND two sets together
		set32_Or()			- OR two sets together
		set32_Xor()			- Exclusive-OR two sets together
		set32_Tst()			- test if any bits are set
		set32_Count()		- count how many bits are set

		set32_Set1()		- set one bit
		set32_Clr1()		- clear one bit
		set32_Not1()		- invert one bit
		set32_Tst1()		- test if a bit is set

 **********************************************************/

#include <set32.h>


/*----------------- Forward References -------------------*/

extern BYTE gSET32_bitcount[];



/**********************************************************

    set32_Count - count how many bits are set

	Description:

	Return value:
		0 to 32

**********************************************************/
BYTE set32_Count
(
	sSET32	bitset		/* I: the bitset to count the '1's of */
)
{
	union
	{
		sSET32	normal;
		BYTE	fourbytes[4];

	} work;

	register BYTE *byte_ptr = work.fourbytes;
	register BYTE *array_ptr = gSET32_bitcount;

	work.normal = bitset;

	return( array_ptr[ *byte_ptr++ ] +
		    array_ptr[ *byte_ptr++ ] +
		    array_ptr[ *byte_ptr++ ] +
		    array_ptr[ *byte_ptr ] );
}




static BYTE gSET32_bitcount[256] =
{
    0,  /*   0 */
    1,  /*   1 */
    1,  /*   2 */
    2,  /*   3 */
    1,  /*   4 */
    2,  /*   5 */
    2,  /*   6 */
    3,  /*   7 */
    1,  /*   8 */
    2,  /*   9 */
    2,  /*  10 */
    3,  /*  11 */
    2,  /*  12 */
    3,  /*  13 */
    3,  /*  14 */
    4,  /*  15 */
    1,  /*  16 */
    2,  /*  17 */
    2,  /*  18 */
    3,  /*  19 */
    2,  /*  20 */
    3,  /*  21 */
    3,  /*  22 */
    4,  /*  23 */
    2,  /*  24 */
    3,  /*  25 */
    3,  /*  26 */
    4,  /*  27 */
    3,  /*  28 */
    4,  /*  29 */
    4,  /*  30 */
    5,  /*  31 */
    1,  /*  32 */
    2,  /*  33 */
    2,  /*  34 */
    3,  /*  35 */
    2,  /*  36 */
    3,  /*  37 */
    3,  /*  38 */
    4,  /*  39 */
    2,  /*  40 */
    3,  /*  41 */
    3,  /*  42 */
    4,  /*  43 */
    3,  /*  44 */
    4,  /*  45 */
    4,  /*  46 */
    5,  /*  47 */
    2,  /*  48 */
    3,  /*  49 */
    3,  /*  50 */
    4,  /*  51 */
    3,  /*  52 */
    4,  /*  53 */
    4,  /*  54 */
    5,  /*  55 */
    3,  /*  56 */
    4,  /*  57 */
    4,  /*  58 */
    5,  /*  59 */
    4,  /*  60 */
    5,  /*  61 */
    5,  /*  62 */
    6,  /*  63 */
    1,  /*  64 */
    2,  /*  65 */
    2,  /*  66 */
    3,  /*  67 */
    2,  /*  68 */
    3,  /*  69 */
    3,  /*  70 */
    4,  /*  71 */
    2,  /*  72 */
    3,  /*  73 */
    3,  /*  74 */
    4,  /*  75 */
    3,  /*  76 */
    4,  /*  77 */
    4,  /*  78 */
    5,  /*  79 */
    2,  /*  80 */
    3,  /*  81 */
    3,  /*  82 */
    4,  /*  83 */
    3,  /*  84 */
    4,  /*  85 */
    4,  /*  86 */
    5,  /*  87 */
    3,  /*  88 */
    4,  /*  89 */
    4,  /*  90 */
    5,  /*  91 */
    4,  /*  92 */
    5,  /*  93 */
    5,  /*  94 */
    6,  /*  95 */
    2,  /*  96 */
    3,  /*  97 */
    3,  /*  98 */
    4,  /*  99 */
    3,  /* 100 */
    4,  /* 101 */
    4,  /* 102 */
    5,  /* 103 */
    3,  /* 104 */
    4,  /* 105 */
    4,  /* 106 */
    5,  /* 107 */
    4,  /* 108 */
    5,  /* 109 */
    5,  /* 110 */
    6,  /* 111 */
    3,  /* 112 */
    4,  /* 113 */
    4,  /* 114 */
    5,  /* 115 */
    4,  /* 116 */
    5,  /* 117 */
    5,  /* 118 */
    6,  /* 119 */
    4,  /* 120 */
    5,  /* 121 */
    5,  /* 122 */
    6,  /* 123 */
    5,  /* 124 */
    6,  /* 125 */
    6,  /* 126 */
    7,  /* 127 */
    1,  /* 128 */
    2,  /* 129 */
    2,  /* 130 */
    3,  /* 131 */
    2,  /* 132 */
    3,  /* 133 */
    3,  /* 134 */
    4,  /* 135 */
    2,  /* 136 */
    3,  /* 137 */
    3,  /* 138 */
    4,  /* 139 */
    3,  /* 140 */
    4,  /* 141 */
    4,  /* 142 */
    5,  /* 143 */
    2,  /* 144 */
    3,  /* 145 */
    3,  /* 146 */
    4,  /* 147 */
    3,  /* 148 */
    4,  /* 149 */
    4,  /* 150 */
    5,  /* 151 */
    3,  /* 152 */
    4,  /* 153 */
    4,  /* 154 */
    5,  /* 155 */
    4,  /* 156 */
    5,  /* 157 */
    5,  /* 158 */
    6,  /* 159 */
    2,  /* 160 */
    3,  /* 161 */
    3,  /* 162 */
    4,  /* 163 */
    3,  /* 164 */
    4,  /* 165 */
    4,  /* 166 */
    5,  /* 167 */
    3,  /* 168 */
    4,  /* 169 */
    4,  /* 170 */
    5,  /* 171 */
    4,  /* 172 */
    5,  /* 173 */
    5,  /* 174 */
    6,  /* 175 */
    3,  /* 176 */
    4,  /* 177 */
    4,  /* 178 */
    5,  /* 179 */
    4,  /* 180 */
    5,  /* 181 */
    5,  /* 182 */
    6,  /* 183 */
    4,  /* 184 */
    5,  /* 185 */
    5,  /* 186 */
    6,  /* 187 */
    5,  /* 188 */
    6,  /* 189 */
    6,  /* 190 */
    7,  /* 191 */
    2,  /* 192 */
    3,  /* 193 */
    3,  /* 194 */
    4,  /* 195 */
    3,  /* 196 */
    4,  /* 197 */
    4,  /* 198 */
    5,  /* 199 */
    3,  /* 200 */
    4,  /* 201 */
    4,  /* 202 */
    5,  /* 203 */
    4,  /* 204 */
    5,  /* 205 */
    5,  /* 206 */
    6,  /* 207 */
    3,  /* 208 */
    4,  /* 209 */
    4,  /* 210 */
    5,  /* 211 */
    4,  /* 212 */
    5,  /* 213 */
    5,  /* 214 */
    6,  /* 215 */
    4,  /* 216 */
    5,  /* 217 */
    5,  /* 218 */
    6,  /* 219 */
    5,  /* 220 */
    6,  /* 221 */
    6,  /* 222 */
    7,  /* 223 */
    3,  /* 224 */
    4,  /* 225 */
    4,  /* 226 */
    5,  /* 227 */
    4,  /* 228 */
    5,  /* 229 */
    5,  /* 230 */
    6,  /* 231 */
    4,  /* 232 */
    5,  /* 233 */
    5,  /* 234 */
    6,  /* 235 */
    5,  /* 236 */
    6,  /* 237 */
    6,  /* 238 */
    7,  /* 239 */
    4,  /* 240 */
    5,  /* 241 */
    5,  /* 242 */
    6,  /* 243 */
    5,  /* 244 */
    6,  /* 245 */
    6,  /* 246 */
    7,  /* 247 */
    5,  /* 248 */
    6,  /* 249 */
    6,  /* 250 */
    7,  /* 251 */
    6,  /* 252 */
    7,  /* 253 */
    7,  /* 254 */
    8   /* 255 */
};
